---
title: Stop squashing your commits
date: 2018-11-20
tags:
  - coding
  - git
draft: true
---

Actually, don't. Or do. Well, actually, it depends.
The question of whether or not you should squash your git commits together is practically a holy war at this point,
but I would like to give my two cents on the issue, if only to clarify my thoughts to myself.

### Background
+ when starting out, people are taught to *commit early, commit often*
When people start learning Git, one of the mantras that they first learn is:

> Commit early, commit often.

The idea is to get people
+ gets used to idea that operations in git are *cheap*, and the advantages gained by telling git about changes
  greatly outweigh the disadvantages.
+ in fact, because all but a few git operations are *local*, you don't need to worry about having everything fleshed
  out before taking advantage of git.
+ implicit in the above is the idea


### Git workflows
+ other people have discussed git workflow, but not really gone into detail about how to craft commits
+ hack until a feature is fully fleshed out, then commit it
  - goes againt *commit early, commit often*
  - don't do this
+ hack until a feature is fully fleshed out, commiting at random
  - git delivers diminishing returns if commits don't correspond to logical changes
    - hard to skip back to a "working" state
    - harder to see what has actually been changed without contextual and correct commit messages 
  - commits can be "at random" to a greater extent, but the idea is that commits are made according
    to some scheme that is divorced from the code (e.g. commiting before you go to lunch, or at the end of
    the day
+ split feature into smaller pieces, then tackle these pieces one at a time, making a single commit for each
  - this is the "ideal", we should strive for this
  - difficult to do in practice because it presupposes that you already understand the problem well enough
    to split it up.

###


The idea is to get people
used to the idea that, when using git, commits are *cheap*. This is in contrast to older version control systems
(VCS) where commiting is often a rather hefty operation, and where the idea of making *several commits a day* is
pretty crazy. The unintended consequence of this mentali

The fundamental point is that *git history is a historical record*. This might seem like a tautology, but
my point is that there are several ways of interpreting git history.


### The git history as an eye-witness account / lab book
History is *what actually happened*.
"from the trenches". You feel like you're there; oh the trailing whitespace; oh the humanity!.
Get accounts from both sides of the battle.

+ immutability is a public good
+ don't get problems when collaborating

### The git history as a historical account / scientific article
History is written by the victors. When writing a scientific article, you have to keep in mind
that the majority of your readers don't care whether you spent several weeks on a particularly
difficult calculation, trying many routes that ended up as dead ends, all they care about is
the *correct* method for obtaining the results.

Of course, it may be that you publish something incorrect, and later have to write another paper
to correct the previous one (rare in the academic world, but very common when writing code). This
is to some extent unavoidable, but it does not diminish the importance of taking the time to distill
down the experience into a digestible portion.

----
*Why are you recording all of history*? Why not use dropbox, or dropbox + some history (say 1 week / 1 month).
Could be several reasons:

+ auditing (knowing who changed what). Might be important for businesses for knowing who to promote/fire
+ improving understanding
+ increased control / granularity

### The git history as a lab book
An immutable record of things "as they went down". Many dead ends and mistakes.
Requires extra work to see what the actual progress was.

### The git history as a journal article
Curated to make the content as understandable as possible. Nobody cares that you spent 3
days tracking down a particularly insidious bug.


### Two distinct modes of operation
+ A personal record
  - a quick backup in case I accidentally `rm` something
  - allows to explore various paths

+ An account for others to understand the decisions that went into making the code.
  If there is something that I don't understand, I will usually grep the git log for that
  change. 

- could use branches as the "article" and individual commits as the "lab book". The message of the merge commit
  should be the same as the message of the single commit in the "article" way of working.
